@@ -408,16 +408,17 @@ sub get_dict {
     }
 
     if( !exists $self->{ $lang } ) {
-	if ( $self->_init_lang( $lang ) ) {
-	    $self->{cur_lang} = $lang;
-	}
-	else {
-	    Activator::Exception::Dictionary->throw( 'init_lang', 'failed' );
+	try eval {
+	    $self->_init_lang( $lang );
+	};
+	if ( catch my $e ) {
+	    Activator::Exception::Dictionary->throw( 'init_lang', 'failed', $e );
 	}
+
     }
-    else {
-	$self->{cur_lang} = $lang;
-    }
+
+    $self->{cur_lang} = $lang;
+
     return $self;
 }
 
@@ -442,7 +443,13 @@ sub new {
 sub _init_config {
     my ($self) = @_;
 
+    # old config format
     my $config = Activator::Registry->get('Activator::Dictionary');
+    if ( !$config ) {
+	# new format
+	$config = Activator::Registry->get('Activator->Dictionary');
+    }
+
 
     $self->{config}->{default_realm} = $config->{default_realm} || 'default';
     $self->{config}->{default_lang}  = $config->{default_lang} || 'en';
@@ -511,7 +518,7 @@ sub _init_lang {
 
 	    # This message could be annoying in some situations, so
 	    # allow changing the log level for just this one.
-	    my $msg = "Couldn't load dictionary from file for $lang";
+	    my $msg = "Couldn't load dictionary from file for $lang from $dir_loc";
 	    my $level = $self->{LOG_LEVEL_FOR_FILE_LOAD};
 	    if ( $level =~ /FATAL|ERROR|WARN|INFO|DEBUG|TRACE/ ) {
 		no strict 'refs';
@@ -567,11 +574,11 @@ L<Exception::Class::TryCatch>, L<Class::StrongSingleton>
 
 =head1 AUTHOR
 
-Karim Nassar
+Karim A. Nassar
 
 =head1 COPYRIGHT
 
-Copyright (c) 2007 Karim Nassar <karim.nassar@acm.org>
+Copyright (c) 2007 Karim A. Nassar <karim.nassar@acm.org>
 
 You may distribute under the terms of either the GNU General Public
 License or the Artistic License, as specified in the Perl README file.
